@@ -35,7 +35,7 @@ def set_seed(seed):
     np.random.seed(seed)
     random.seed(seed)
 
-def run_training(rank, world_size, model_args, data, load_from, new, num_train_steps, name, seed):
+def run_training(rank, world_size, model_args, data, load_from, new, num_train_steps, name, seed, use_aim, aim_repo, aim_run_hash):
     is_main = rank == 0
     is_ddp = world_size > 1
 
@@ -53,7 +53,7 @@ def run_training(rank, world_size, model_args, data, load_from, new, num_train_s
         world_size = world_size
     )
 
-    model = Trainer(**model_args)
+    model = Trainer(**model_args, hparams=model_args, use_aim=use_aim, aim_repo=aim_repo, aim_run_hash=aim_run_hash)
 
     if not new:
         model.load(load_from)
@@ -116,6 +116,9 @@ def train_from_folder(
     seed = 42,
     amp = False,
     show_progress = False,
+    use_aim = True,
+    aim_repo = None,
+    aim_run_hash = None
 ):
     num_image_tiles = default(num_image_tiles, 4 if image_size > 512 else 8)
 
@@ -178,11 +181,11 @@ def train_from_folder(
     world_size = torch.cuda.device_count()
 
     if world_size == 1 or not multi_gpus:
-        run_training(0, 1, model_args, data, load_from, new, num_train_steps, name, seed)
+        run_training(0, 1, model_args, data, load_from, new, num_train_steps, name, seed, use_aim, aim_repo, aim_run_hash)
         return
 
     mp.spawn(run_training,
-        args=(world_size, model_args, data, load_from, new, num_train_steps, name, seed),
+        args=(world_size, model_args, data, load_from, new, num_train_steps, name, seed, use_aim, aim_repo, aim_run_hash,),
         nprocs=world_size,
         join=True)
 
